﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Actions
{
    /// <summary>
    /// Class that contains <c>ICardAction</c>s for game, that can user use
    /// </summary>
    class ActionContainer : IActionContainer
    {
        /// <summary>
        /// List of all <c>ICardAction</c>s
        /// </summary>
        List<ICardAction> _Actions;

        /// <summary>
        /// Creates new <c>ActionContainer</c> wich will contain Actions passed as param
        /// </summary>
        /// <param name="Actions">Array of <c>ICardAction</c>s</param>
        public ActionContainer(ICardAction[] Actions)
        {
            this._Actions = new List<ICardAction>(Actions);
        }

        public ICardAction[] Actions
        {
            get
            {
                return _Actions.ToArray();
            }
        }

        public int[] AcceptableIDs
        {
            get
            {
                return _Actions
                        .SkipWhile( card => card.CallingID == -1)
                        .Select( x => x.CallingID)
                        .ToArray();
            }
        }

        public ICardAction GetActionWithID(int ID)
        {
            if (AcceptableIDs.Contains(ID))
                foreach (ICardAction Action in _Actions)
                    if (Action.CallingID == ID)
                        return Action;
            Exception e = new Exception("Something went wrong in ActionContainer.GetActionWithId");
            e.Data["EndState"] = ProgramsEndStateEnum.ActionContainerUnexpectedId;
            throw e;
        }
    }
}
