﻿using Prsi.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Actions
{
    class ActionFactory : IActionFactory
    {

        public ICardAction[] CreateActions(IStack Stack, ICard[] PlayerCards)
        {
            int index = 0;
            List<ICardAction> Actions = CreateCardsActions(Stack.LastCard,PlayerCards,out index);
            
            if (Stack.LastCard.HasSpecialAction)
                Actions.Add(Stack.LastCard.SpecialAction(++index));

            if (PlayerCards.Length == 0 && !Stack.LastCard.CanTakePlayerBackToGame)
                Actions.Add(new CardAction("Win", Win, ++index, null));

            if (Stack.CanDrawCard && Stack.LastCard.CanDrawFromStack)
                Actions.Add(new CardAction("Draw card", DrawCard, ++index, null));

            if (Actions.Count == 0 && !Stack.CanDrawCard)
                Actions.Add(new CardAction("Skip turn", SkipTurn, ++index, null));

            ICardAction[] ActionsInArray = Actions.ToArray();
            ActionsIDValidation(ActionsInArray);
            return ActionsInArray;
        }

        private void ActionsIDValidation(ICardAction[] Actions)
        {
            List<int> Ids = new List<int>(Actions.Length);
            foreach (ICardAction Action in Actions)
            {
                if (Action.CallingID == -1)
                    continue;
                if (Ids.Contains(Action.CallingID))
                {
                    Exception e = new Exception("There were discoveres more actions with same ID");
                    e.Data["EndState"] = ProgramsEndStateEnum.MoreActionsWithOneID;
                    throw e;
                }
                Ids.Add(Action.CallingID);
            }
        }

        private List<ICardAction> CreateCardsActions(ICard CardOnStack, ICard[] PlayerCards, out int Index)
        {
            int FuncIndex = 0;
            List<ICardAction> Actions = new List<ICardAction>();
            foreach (ICard ActualCard in PlayerCards)
            {
                string CardBrand = string.Format("{0}{1} {2} {3}", (char)ActualCard.Color, (char)ActualCard.Value,
                                                                  ActualCard.Color, ActualCard.Value);
                if (CardOnStack.CanBePlaced(ActualCard))
                    Actions.Add(new CardAction(CardBrand, ActualCard.ActionToPerform, ++FuncIndex, ActualCard));
                else
                    Actions.Add(new CardAction(CardBrand));
            }
            Index = FuncIndex;
            return Actions;
        }


        #region ActionsUnrelatedToCards
        private static bool DrawCard(IStack Stack, IPlayer Player, ICard Card)
        {
            Player.TakeCard(Stack.DrawCard());
            return false;
        }
        private static bool Win(IStack Stack, IPlayer Player, ICard Card)
        {
            return true;
        }
        private static bool SkipTurn(IStack Stack, IPlayer Player, ICard Card)
        {
            return false;
        }
        #endregion
    }
}
