﻿using Prsi.ViewModels;
using Prsi.ViewModels.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi
{
    /// <summary>
    /// Represent class, what will load config from console
    /// </summary>
    class Config : IConfig
    {
        IConfigViewModel _Input;
        int _CountOfPlayers;
        bool _EnhancedSet = false;
        int _AmountOfCardForEachPlayer;
        bool _WasConfigurationLoaded = false;

        /// <summary>
        /// Creates new instance
        /// </summary>
        /// <param name="Input">ViewModel which will be use for input</param>
        public Config(IConfigViewModel Input)
        {
            _Input = Input;
        }


        public void LoadConfiguration()
        {
            _CountOfPlayers = _Input.CountOfPlayers;
            _AmountOfCardForEachPlayer = _Input.AmountOfCardForEachPlayer;
            if (_CountOfPlayers > 4)
                _EnhancedSet = _Input.IsEnhancedSetOfCards;
            _WasConfigurationLoaded = true;
        }

        public int CountOfPlayers
        {
            get 
            {
                if (_WasConfigurationLoaded)
                    return _CountOfPlayers;
                ThrowConfigurationWasntLoaded();
                return -1;
            }
        }

        public bool UseEnhancedSet
        {
            get
            {
                if (_WasConfigurationLoaded)
                    return _EnhancedSet;
                ThrowConfigurationWasntLoaded();
                return false;
            }
        }

        public int AmountOfCardForEachPlayer
        {
            get
            {
                if (_WasConfigurationLoaded)
                    return _AmountOfCardForEachPlayer;
                ThrowConfigurationWasntLoaded();
                return -1;
            }
        }

        private void ThrowConfigurationWasntLoaded()
        {
            Exception e = new Exception("Configuration wasnt loaded");
            e.Data["EndState"] = ProgramsEndStateEnum.ConfigurationWasntLoaded;
            throw e;
        }
    }
}
