﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.ConsoleOperation
{
    /// <summary>
    /// This class will automatically convert input
    /// </summary>
    class FromConsoleConverter
    {
        private IMyConsole Input;

        /// <summary>
        /// Creates new instance
        /// </summary>
        /// <param name="_Input">Stream, from which will instance takes input</param>
        public FromConsoleConverter(IMyConsole Input)
        {
            this.Input = Input;
        }

        /// <summary>
        /// Convert input
        /// </summary>
        /// <typeparam name="T">type to convert</typeparam>
        /// <param name="Converter">callback which will convert from string to <c>T</c></param>
        /// <returns>Return type converter from string</returns>
        /// <exception>Can throw same exceptions as Converter</exception>
        public T ConvertInput<T>(Func<string,T> Converter)
        {
            return Converter(Input.Text);
        }

        /// <summary>
        /// Convert input, but first will ask for question.
        /// </summary>
        /// <typeparam name="T">type to convert</typeparam>
        /// <param name="Converter">callback, which will convert from string to <c>T</c></param>
        /// <param name="Question">question to ask every time</param>
        /// <param name="Formater">callback to format question</param>
        /// <returns>Return type converter from string</returns>
        /// <exception>this function cannot throw exception</exception>
        public T ConvertInput<T>(Func<string,T> Converter,string Question, Func<string,string> Formater)
        {
            while(true)
            {
                try
                {
                    Input.Text = Formater(Question);
                    string input = Input.Text; 
                    T Result = Converter(input);
                    return Result;
                }
                catch(Exception)
                {
                    Input.Text = Formater("Invalid data, please repeat");
                }
            }
        }

        /// <summary>
        /// Convert input, but first will ask for question. Value will be validat.
        /// </summary>
        /// <typeparam name="T">type to convert</typeparam>
        /// <param name="Converter">callback, which will convert from string to <c>T</c></param>
        /// <param name="Question">question to ask every time</param>
        /// <param name="Formater">callback to format question</param>
        /// <param name="Validation">callback, which must return true if is value valid</param>
        /// <returns>Return type converter from string</returns>
        /// <exception>this function cannot throw exception</exception>
        public T ConvertInput<T>(Func<string,T> Converter,string Question, Func<string,string> Formater,Func<T,bool> Validation)
        {
            T Value;
            while(true)
            {
                Value = ConvertInput(Converter,Question,Formater);
                if (!Validation(Value))
                    Input.Text = Formater("Value is out of range, please repeat");
                else
                    return Value;
            }
        }
    }
}
