﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.ConsoleOperation
{
    class OutputFormater : IOutputFormater
    {
        IMyConsole Input;

        public OutputFormater(IMyConsole Input)
        {
            this.Input = Input;
        }

        #region TextGenerating
        public string AlignLeft(string Text) { return this.AlignLeft(Text,' '); }
        public string AlignLeft(string Text, char FillSymbol) { return this.AlignLeft(Text,0,FillSymbol); }
        public string AlignLeft(string Text, int Width) { return this.AlignLeft(Text, Width, ' '); }
        public string AlignLeft(string Text, int Width, char FillingSymbol)
        {
            Width = ValidateWidth(Text, Width);
            int SpaceLeft = 0;
            int SpaceRight = Width - Text.Length;
            return GenerateText(Text, SpaceLeft, SpaceRight, FillingSymbol);
        }

        public string AlignRight(string Text) { return this.AlignRight(Text, ' '); }
        public string AlignRight(string Text, char FillSymbol) { return this.AlignRight(Text, 0, FillSymbol); }
        public string AlignRight(string Text, int Width) { return this.AlignRight(Text, Width, ' '); }
        public string AlignRight(string Text, int Width, char FillingSymbol)
        {
            Width = ValidateWidth(Text, Width);
            int SpaceLeft = Width - Text.Length;
            int SpaceRight = 0;
            return GenerateText(Text, SpaceLeft, SpaceRight, FillingSymbol);
        }

        public string AlignCenter(string Text) { return this.AlignCenter(Text, ' '); }
        public string AlignCenter(string Text, char FillSymbol) { return this.AlignCenter(Text, 0, FillSymbol); }
        public string AlignCenter(string Text, int Width) { return this.AlignCenter(Text, Width, ' '); }
        public string AlignCenter(string Text, int Width, char FillingSymbol)
        {
            Width = ValidateWidthIfZero(Width);
            List<string> Lines = ParseTextsToCenter(Text,Width);
            StringBuilder Builder = new StringBuilder();
            foreach (string Line in Lines)
            {
                int SpaceLeft = (Width - Line.Length) / 2;
                int SpaceRight = Width - Line.Length - SpaceLeft;
                string CentralizedText = GenerateText(Line, SpaceLeft, SpaceRight, FillingSymbol);
                Builder.Append(AlignLeft(CentralizedText,' '));
            }
            return Builder.ToString();
        }

        private List<string> ParseTextsToCenter(string Line,int Width)
        {
            List<string> OutputList = new List<string>();
            if(Line.Length<=Width)
            {
                OutputList.Add(Line);
                return OutputList;
            }
            int Index = Line.LastIndexOf(' ',Width);
            string LeftPart=Line.Substring(0,Index);
            string RightPart=Line.Substring(Index+1);
            OutputList.InsertRange(0, ParseTextsToCenter(RightPart, Width));
            OutputList.InsertRange(0, ParseTextsToCenter(LeftPart, Width));
            return OutputList;

        }

        private string GenerateText(string Text, int SpaceLeft, int SpaceRight, char FillinfSymbol)
        {
            StringBuilder Builder = new StringBuilder(Text.Length + SpaceLeft + SpaceRight);
            Builder.Append(FillinfSymbol, SpaceLeft);
            Builder.Append(Text);
            Builder.Append(FillinfSymbol, SpaceRight);
            return Builder.ToString();
        }
        #endregion

        #region WidthValidation
        private int ValidateWidth(string Text, int OldWidth)
        {
            OldWidth = ValidateWidthIfZero(OldWidth);
            OldWidth = ValidateWidthAgainTextLength(Text, OldWidth);
            return OldWidth;
        }

        private int ValidateWidthAgainTextLength(string Text, int Width)
        {
            while (Width == Input.Width && Width < Text.Length)
                Width += Input.Width;
            if (Width < Text.Length)
                Width = Text.Length;
            return Width;
        }
        private int ValidateWidthIfZero(int OldWidth)
        {
            if (OldWidth <= 0)
                return Input.Width;
            else
                return OldWidth;
        }
        #endregion

        public void Clear()
        {
            Input.Clear();
        }
    }
}
