﻿using Prsi.Actions;
using Prsi.Entities.CardsStates;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Entities
{
    class Card : IConfigurableCard
    {
        private CardColor _Color;
        private CardValues _Value;
        private ICardState _ActualState;
        private IDictionary<CardStatesEnum, ICardState> _States;
        

        public Card(CardColor Color, CardValues Value)
        {
            _Color = Color;
            _Value = Value;
            _States = new Dictionary<CardStatesEnum,ICardState>();
        }

        public CardColor Color
        {
            get { return _Color; }
        }

        public CardValues Value
        {
            get { return _Value; }
        }


        public bool AreEqual(ICard Second)
        {
            return _Color == Second.Color && _Value == Second.Value;
        }


        #region StateFuncs
        public bool CanBePlaced(ICard Card)
        {
            return _ActualState.CanBePlaced(this,Card);
        }

        public Func<IStack, IPlayer,ICard,bool> ActionToPerform
        {
            get 
            {
                IConfigurableCard Card = this;
                return _ActualState.ActionToPerform; 
            }
        }

        public bool CanDrawFromStack
        {
            get { return _ActualState.CanDrawFromStack; }
        }

        public void ActionAfterPutedOnStack()
        {
            _ActualState.ActionAfterPutedOnStack();
        }

        public bool CanTakePlayerBackToGame
        {
            get { return _ActualState.CanTakePlayerBackToGame; }
        }
        public bool HasSpecialAction
        {
            get { return _ActualState.HasSpecialAction; }
        }

        public ICardAction SpecialAction(int index)
        {
            return _ActualState.SpecialAction(index);
        }
        #endregion

        #region WorkingWithStates
        public void SetState(CardStatesEnum State)
        {
            _ActualState = _States[State];
        }


        public void SetDefaultState()
        {
            SetState(DefaultState);
        }

        public void AddState(ICardState State)
        {
            if(!_States.Contains(new KeyValuePair<CardStatesEnum,ICardState>(State.ID,State)))
            {
                _States.Add(State.ID,State);
            }
            else
            {
                Exception e = new Exception("This state is already in card");
                e.Data["EndState"] = ProgramsEndStateEnum.DupliciteStates;
                throw e;
            }
        }

        public CardStatesEnum DefaultState { get; set; }
        #endregion
    }
}
