﻿using Prsi.Actions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Entities.CardsStates
{
    /// <summary>
    /// Represent state, from which will be inherited
    /// </summary>
    abstract class BaseCardState : ICardState
    {
        protected IConfigurableCard _Card;
        public BaseCardState(IConfigurableCard Card)
        {
            _Card = Card;
        }

        public abstract bool CanBePlaced(ICard First, ICard Second);

        public virtual Func<IStack, IPlayer, ICard, bool> ActionToPerform
        {
            get 
            {
                return FuncActionNextTurn;
            }
        }

        public virtual bool CanDrawFromStack
        {
            get { return true; }
        }

        public virtual void ActionAfterPutedOnStack()
        {
            
        }

        public virtual bool CanTakePlayerBackToGame
        {
            get { return false; }
        }

        private static bool FuncActionNextTurn(IStack Stack,IPlayer Player,ICard Card)
        {
            Player.PutCard(Card);
            Stack.LastCard.SetDefaultState();
            Stack.PutCard(Card);
            return false;
        }


        public virtual bool HasSpecialAction
        {
            get { return false; }
        }

        public virtual ICardAction SpecialAction(int index)
        {
            return null;
        }

        public abstract CardStatesEnum ID { get; }
    }
}
