﻿using Prsi.Actions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Entities.CardsStates
{
    class TakingSevenState : BaseCardState
    {
        Taker _CardTaker;

        public TakingSevenState(IConfigurableCard Card,Taker CardTaker) : base(Card) 
        {
            this._CardTaker = CardTaker;
        }

        public override bool CanBePlaced(ICard First, ICard Second)
        {
            bool IsKing = Second.Value == CardValues.King && Second.Color == CardColor.Spade;
            bool IsThisSpadeSeven = First.Color == CardColor.Spade;
            return First.Value == Second.Value || (IsThisSpadeSeven && IsKing);
        }

        public override Func<IStack, IPlayer, ICard, bool> ActionToPerform
        {
            get
            {
                return FuncActionNextTurn;
            }
        }

        public override bool CanDrawFromStack
        {
            get { return false; }
        }

        public override void ActionAfterPutedOnStack()
        {
            _CardTaker.CardsToTake += 2;
        }

        public override bool CanTakePlayerBackToGame
        {
            get { return true; }
        }

        private static bool FuncActionNextTurn(IStack Stack, IPlayer Player, ICard Card)
        {
            Player.PutCard(Card);
            Stack.LastCard.SetDefaultState();
            Stack.PutCard(Card);
            return false;
        }


        public override bool HasSpecialAction
        {
            get { return true; }
        }

        public override ICardAction SpecialAction(int index)
        {
            return new CardAction(string.Format("Take {0} cards", _CardTaker.CardsToTake), Take2Cards, index, _Card);
        }

        private bool Take2Cards(IStack Stack, IPlayer Player, ICard Card)
        {
            for (int a = 0; a < _CardTaker.CardsToTake; a++)
                if (Stack.CanDrawCard)
                    Player.TakeCard(Stack.DrawCard());
            _Card.SetState(CardStatesEnum.UsedSevenState);
            _CardTaker.CardsToTake = 0;
            return false;
        }

        public override CardStatesEnum ID
        {
            get { return CardStatesEnum.TakingSevenState; }
        }
    }
}
