﻿using Prsi.Entities.CardsStates;
using Prsi.ViewModels.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Entities
{
    class EntitiesFactory : IEntitiesFactory
    {
        private IFactoryViewModel ViewModel;
        private List<IConfigurablePlayer> _Players;
        private IConfigurableStack _Stack;

        Taker _CardTaker = new Taker();

        public EntitiesFactory(IFactoryViewModel ViewModel)
        {
            this.ViewModel = ViewModel;
            _Players = new List<IConfigurablePlayer>();
        }

        public bool CreatePlayers(int CountOfPlayers)
        {
            for (int a = 0; a < CountOfPlayers; a++)
                _Players.Add(ViewModel.CreatePlayer(a+1));
            return true;
        }

        public bool CreateStack(bool UseEnhancedSet)
        {
            List<ICard> Cards = CreateSetOfCard();
            if (UseEnhancedSet)
                Cards.AddRange(CreateSetOfCard());

            _Stack = new Stack(Cards.ToArray());
            _Stack.MixCards();

            return true;
        }

        public void GiveCardsToPlayer(int CountOfCards)
        {
            if (CountOfCards * _Players.Count > _Stack.GetCountOfCards)
            {
                Exception e = new Exception("Count of cards is smaller that player needs");
                e.Data["EndState"] = ProgramsEndStateEnum.NotEnoughtCards;
                throw e;
            }
            for (int a = 0; a < CountOfCards;a++ )
            {
                foreach(IConfigurablePlayer Player in _Players)
                    Player.TakeCard(_Stack.DrawCard());
            }
                

        }

        public IPlayer[] Players
        {
            get 
            {
                Random Random = new Random();
                return _Players.OrderBy((x) => { return Random.Next(); }).ToArray();
            }

        }

        public IStack Stack
        {
            get { return _Stack; }
        }

        private List<ICard> CreateSetOfCard()
        {
            List<ICard> Cards = new List<ICard>(32);
            foreach(CardColor Color in Enum.GetValues(typeof(CardColor)))
            {
                foreach(CardValues Value in Enum.GetValues(typeof(CardValues)))
                {
                    IConfigurableCard Card = new Card(Color, Value);
                    AssignStates(Card);
                    Cards.Add(Card);
                }
            }
            return Cards;
        }

        private void AssignStates(IConfigurableCard Card)
        {
            if (Card.Value == CardValues.King && Card.Color == CardColor.Spade)
            {
                Card.AddState(new TakingSpadeKingState(Card,_CardTaker));
                Card.AddState(new UsedSpadeKingState(Card));
                Card.DefaultState = CardStatesEnum.UsedSpadeKingState;
            }
            else if (Card.Value == CardValues.Seven)
            {
                Card.AddState(new TakingSevenState(Card,_CardTaker));
                Card.AddState(new UsedSevenState(Card));
                Card.DefaultState = CardStatesEnum.UsedSevenState;
            }
            else if (Card.Value == CardValues.Ace)
            {
                Card.AddState(new WaitingAceState(Card));
                Card.AddState(new UsedAceState(Card));
                Card.DefaultState = CardStatesEnum.UsedAceState;
            }
            else
            {
                Card.AddState(new NormalCardState(Card));
                Card.DefaultState = CardStatesEnum.NormalCardState;
            }

            Card.SetDefaultState();
        }
    }
}
