﻿using Prsi.Actions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi.Entities
{
    /// <summary>
    /// Interface that is same for CardState and Card. Card call same method of CardState.
    /// </summary>
    interface ICardAndCardStateShared
    {
        /// <summary>
        /// Get actions to perform when have it player in hand
        /// </summary>
        Func<IStack, IPlayer, ICard, bool> ActionToPerform { get; }

        /// <summary>
        /// True if player can draw and this card is on stack
        /// </summary>
        bool CanDrawFromStack { get; }

        /// <summary>
        /// This action will be call after card is place on stack
        /// </summary>
        void ActionAfterPutedOnStack();

        /// <summary>
        /// Return true, if has Card special efect when is on top of stack
        /// </summary>
        bool HasSpecialAction { get; }

        /// <summary>
        /// Return action, which have effect when is card on the top of stack
        /// </summary>
        /// <param name="index">Index which will be passed to ICardAction</param>
        /// <returns></returns>
        ICardAction SpecialAction(int index);

        /// <summary>
        /// Return true, if could this player return player back to game when is on the top of stack
        /// </summary>
        bool CanTakePlayerBackToGame { get; }
    }
}
