﻿using Prsi.Entities;
using Prsi.ViewModels.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi
{
    /// <summary>
    /// Class organize game, players queue and winners
    /// </summary>
    class GameOrganizer
    {
        Queue<IPlayer> _Players;
        Queue<IPlayer> _Winners;
        IPlayer _ActualPlayer;

        /// <summary>
        /// Create new instance
        /// </summary>
        /// <param name="Players">Player ranked by order in game</param>
        public GameOrganizer(IPlayer[] Players)
        {
            _Players = new Queue<IPlayer>(Players);
            _Winners = new Queue<IPlayer>(Players.Length-1);
            _ActualPlayer = _Players.Dequeue();
        }

        /// <summary>
        /// Get players except that one on his turn
        /// </summary>
        public Queue<IPlayer> OtherPlayers
        {
            get
            {
                return new Queue<IPlayer>(_Players);
            }
        }

        /// <summary>
        /// Return player on his turn
        /// </summary>
        public IPlayer ActualPlayer
        {
            get
            {
                return _ActualPlayer;
            }
        }

        /// <summary>
        /// Check, if the game should end
        /// </summary>
        public bool EndGame
        {
            get
            {
                //0 because the last one (ActualPlayer) is in variable, not in queue
                return _Players.Count == 0;
            }
        }

        /// <summary>
        /// Will add actual player back to queue and take next player
        /// </summary>
        /// <param name="Call">callback to call in the end</param>
        public void NextTurn(Action Call=null)
        {
            if(_ActualPlayer!=null)
                _Players.Enqueue(_ActualPlayer);
            _ActualPlayer = _Players.Dequeue();
            if (Call != null)
                Call();
        }

        /// <summary>
        /// Remove player from game queue and add it to player
        /// </summary>
        /// <param name="Player">Winner player</param>
        public void AddWinner(IPlayer Player)
        {
            _Winners.Enqueue(Player);
            if (Player == _ActualPlayer)
                _ActualPlayer = null;
            else
            {
                Exception e = new Exception("Cannot add to winners different players that is on his turn");
                e.Data["EndState"] = ProgramsEndStateEnum.AddedPlayerNotOnRound;
                throw e;
            }
        }

        /// <summary>
        /// Return array of winner, ordered by time when they win
        /// </summary>
        public IPlayer[] Winners
        {
            get
            {
                return _Winners.ToArray();
            }
        }
    }
}
