﻿using Prsi.Actions;
using Prsi.ConsoleOperation;
using Prsi.Entities;
using Prsi.ViewModels;
using Prsi.ViewModels.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Prsi
{
    /// <summary>
    /// Contains main logic of the game
    /// </summary>
    class Prsi
    {
        private IViewModel _ViewModel;

        /// <summary>
        /// Creates new instance
        /// </summary>
        /// <param name="ViewModel">ViewModel throught which will all informations be showed</param>
        public Prsi(IViewModel ViewModel)
        {
            _ViewModel = ViewModel;
        }

        /// <summary>
        /// Start the game
        /// </summary>
        /// <returns>Return state of program</returns>
        public int Run()
        {
            // Mainly just prepare game to run
            _ViewModel.WelcomeBye.Welcome();

            IConfig Configuration = new Config(_ViewModel.Configure);
            Configuration.LoadConfiguration();

            IEntitiesFactory Factory = new EntitiesFactory(_ViewModel.Factory);
            Factory.CreatePlayers(Configuration.CountOfPlayers);
            Factory.CreateStack(Configuration.UseEnhancedSet);
            Factory.GiveCardsToPlayer(Configuration.AmountOfCardForEachPlayer);

            StartGame(Factory.Stack,Factory.Players);

            _ViewModel.WelcomeBye.Bye();
            return (int)ProgramsEndStateEnum.NormalReturnState;
        }

        /// <summary>
        /// Start game loop
        /// </summary>
        /// <param name="Stack">Stack wich will be used for game</param>
        /// <param name="Players">Players ranked by order</param>
        protected void StartGame(IStack Stack, IPlayer[] Players)
        {
            GameOrganizer Organizer = new GameOrganizer(Players);
            IActionFactory ActionCreator = new ActionFactory();

            while(!Organizer.EndGame)
            {
                IPlayer Actual = Organizer.ActualPlayer;
                Actual.Login(_ViewModel.Player);

                _ViewModel.Game.ShowActualPlayer(Actual);
                _ViewModel.Game.ShowPlayersStats(Organizer.OtherPlayers,_ViewModel.Player);
                _ViewModel.Game.ShowPlayQueue(Organizer.OtherPlayers);
                _ViewModel.Game.ShowCardOnStack(Stack.LastCard);

                IActionContainer ActionsContainer=new ActionContainer(ActionCreator.CreateActions(Stack,Actual.Cards));
                _ViewModel.Action.DisplayAction(ActionsContainer.Actions);
                int ActionID = _ViewModel.Action.AskForAction(ActionsContainer.AcceptableIDs);

                if (ActionsContainer.GetActionWithID(ActionID).Call(Stack, Actual))
                    Organizer.AddWinner(Actual);
                                  
                Organizer.NextTurn(_ViewModel.Game.ClearOutput);
            }

            _ViewModel.Game.ShowWinners(Organizer.Winners);
        }
    }
}
