﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Prsi.ConsoleOperation;

using PrsiTests.ProgramAbstraction.ConsoleManipulation;

namespace PrsiTests
{
    [TestClass]
    public class ConsoleConvertTest
    {
        #region ConvertOnlyText
        [TestMethod]
        public void ConvertToInt()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("7");
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            int result=Converter.ConvertInput(int.Parse);
            Assert.AreEqual(result,7);
        }

        [TestMethod]
        public void ConvertToSignedInt()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("-27");
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            int result = Converter.ConvertInput(int.Parse);
            Assert.AreEqual(result, -27);
        }

        [TestMethod]
        public void ConvertToFloat()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("7,684");
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            float result = Converter.ConvertInput(float.Parse);
            Assert.AreEqual(result, 7.684,0.0001);
        }

        [TestMethod]
        [ExpectedException(typeof(FormatException))]
        public void ConvertInvalidInt()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("u");
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            int result = Converter.ConvertInput(int.Parse);
        }
        #endregion

        #region ConvertWithQuestionAndFormater
        [TestMethod]
        public void ConvertToIntQ()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("24");
            OutputFormaterAbstract Formater=new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse,Question,Formater.AlignLeft);
            Assert.AreEqual(result, 24);
            Assert.AreEqual(Question, Input.GetOutput(0));
        }

        [TestMethod]
        public void ConvertToSignedIntQ()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("-24");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse, Question, Formater.AlignLeft);
            Assert.AreEqual(result, -24);
            Assert.AreEqual(Question, Input.GetOutput(0));
        }

        [TestMethod]
        public void ConvertToFloatQ()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("24,25");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            float result = Converter.ConvertInput(float.Parse, Question, Formater.AlignLeft);
            Assert.AreEqual(24.25, result ,0.01);
            Assert.AreEqual(Question, Input.GetOutput(0) );
        }

        [TestMethod]
        public void ConvertToIntInvalidQ()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("u");
            Input.AddTextToReturn("v");
            Input.AddTextToReturn("7");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse, Question, Formater.AlignLeft);
            Assert.AreEqual(result, 7);
            Assert.AreEqual(Question,Input.GetOutput(0));
            Assert.AreEqual("Invalid data, please repeat", Input.GetOutput(1));
            Assert.AreEqual(Question, Input.GetOutput(2));
            Assert.AreEqual("Invalid data, please repeat", Input.GetOutput(3));
            Assert.AreEqual(Question, Input.GetOutput(4));
        }
        #endregion

        #region ConvertWithValidation
        [TestMethod]
        public void ConvertToIntV()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("24");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse, Question, Formater.AlignLeft, (number) => { return true; });
            Assert.AreEqual(result, 24);
            Assert.AreEqual(Question, Input.GetOutput(0));
        }

        [TestMethod]
        public void ConvertToSignedIntV()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("-24");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse, Question, Formater.AlignLeft, (number) => { return true; });
            Assert.AreEqual(result, -24);
            Assert.AreEqual(Question, Input.GetOutput(0));
        }

        [TestMethod]
        public void ConvertToFloatV()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("24,25");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            float result = Converter.ConvertInput(float.Parse, Question, Formater.AlignLeft, (number) => { return true; });
            Assert.AreEqual(24.25, result, 0.01);
            Assert.AreEqual(Question, Input.GetOutput(0));
        }

        [TestMethod]
        public void ConvertToIntInvalidV()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("u");
            Input.AddTextToReturn("v");
            Input.AddTextToReturn("7");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse, Question, Formater.AlignLeft, (number) => { return true; });
            Assert.AreEqual(result, 7);
            Assert.AreEqual(Question, Input.GetOutput(0));
            Assert.AreEqual("Invalid data, please repeat", Input.GetOutput(1));
            Assert.AreEqual(Question, Input.GetOutput(2));
            Assert.AreEqual("Invalid data, please repeat", Input.GetOutput(3));
            Assert.AreEqual(Question, Input.GetOutput(4));
        }

        [TestMethod]
        public void ConvertToIntWorkingValidationV()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("24");
            Input.AddTextToReturn("10");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            int result = Converter.ConvertInput(int.Parse, Question, Formater.AlignLeft, 
                                                     (number) => { return number == 10; });
            Assert.AreEqual(result, 10);
            Assert.AreEqual(Question, Input.GetOutput(0));
            Assert.AreEqual("Value is out of range, please repeat", Input.GetOutput(1));
            Assert.AreEqual(Question, Input.GetOutput(2));
        }

        [TestMethod]
        public void ConvertToFloatValidationV()
        {
            InputAbstract Input = new InputAbstract();
            Input.AddTextToReturn("24,25");
            Input.AddTextToReturn("12,64");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Input);
            FromConsoleConverter Converter = new FromConsoleConverter(Input);
            string Question = "Set your number";
            float result = Converter.ConvertInput(float.Parse, Question, Formater.AlignLeft, 
                                                          (number) => { return number > 12.63 && number < 12.65; });
            Assert.AreEqual(12.64, result, 0.01);
            Assert.AreEqual(Question, Input.GetOutput(0));
            Assert.AreEqual("Value is out of range, please repeat", Input.GetOutput(1));
            Assert.AreEqual(Question, Input.GetOutput(2));
        }
        #endregion
    }
}
