﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using PrsiTests.ProgramAbstraction.ConsoleManipulation;
using Prsi.ConsoleOperation;

namespace PrsiTests
{
    [TestClass]
    public class OutputFormaterTest
    {
        #region AlignLeftTests
        [TestMethod]
        public void FormatToLeftWidthoutParam()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignLeft("My own text");
            Assert.AreEqual(Result,"My own text         ");
        }

        [TestMethod]
        public void FormatToLeftSameWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;
            string text = "My own text";

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignLeft(text, text.Length);
            Assert.AreEqual(Result, "My own text");
        }

        [TestMethod]
        public void FormatToLeftWithWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignLeft("My own text",15);
            Assert.AreEqual(Result, "My own text    ");
        }

        [TestMethod]
        public void FormatToLeftWithSymbol()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignLeft("My own text", '#');
            Assert.AreEqual(Result, "My own text#########");
        }

        [TestMethod]
        public void FormatToLeftWithSymbolAndWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignLeft("My own text", 15, '#');
            Assert.AreEqual(Result, "My own text####");
        }
        #endregion

        #region AlignRightTests
        [TestMethod]
        public void FormatToRightWidthoutParam()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignRight("My own text");
            Assert.AreEqual(Result, "         My own text");
        }

        [TestMethod]
        public void FormatToRightSameWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;
            string text = "My own text";

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignRight(text, text.Length);
            Assert.AreEqual(Result, "My own text");
        }

        [TestMethod]
        public void FormatToRightWithWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignRight("My own text", 15);
            Assert.AreEqual(Result, "    My own text");
        }

        [TestMethod]
        public void FormatToRightWithSymbol()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignRight("My own text", '#');
            Assert.AreEqual(Result, "#########My own text");
        }

        [TestMethod]
        public void FormatToRightWithSymbolAndWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignRight("My own text", 15, '#');
            Assert.AreEqual(Result, "####My own text");
        }
        #endregion

        #region AlignCenterTests
        [TestMethod]
        public void FormatToCenterWithoutParametres11letters20width()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("My own text");
            Assert.AreEqual(Result, "    My own text     ");
        }                          //----           -----

        [TestMethod]
        public void FormatToCenterSameWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 15;
            string text = "My own text";

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter(text, text.Length);
            Assert.AreEqual(Result, "My own text    ");
        }

        [TestMethod]
        public void FormatToCenterWithWidth11letters19width()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("My own text",19);
            Assert.AreEqual(Result, "    My own text     ");
        }                          //----           ----_

        [TestMethod]
        public void FormatToCenterWithFillSymbol11letters20width()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("My own text",'#');
            Assert.AreEqual(Result, "####My own text#####");
        }

        [TestMethod]
        public void FormatToCenterWithFillSymbolAndWidth11letters19width()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 20;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("My own text", 19, '#');
            Assert.AreEqual(Result, "####My own text#### ");
        }

        [TestMethod]
        public void FormatToCenterLongTextWithoutWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 10;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("Hello World!!");
            string Expected = "  Hello    World!!  ";
                          //   --Hello----World!!--
                          //             New Line
            Assert.AreEqual(Expected, Result);
        }               

        [TestMethod]
        public void FormatToCenterLongTextWithWidth()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 12;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("Hello World!!",8);
            string Expected = " Hello      World!!     ";
                          //   -Hello--____World!!-____
                          //               New Line
            Assert.AreEqual(Expected, Result);
        }

        [TestMethod]
        public void FormatToCenterLongTextWithWidthAndFillSymbol()
        {
            InputAbstract Console = new InputAbstract();
            Console.Width = 12;

            OutputFormater Formater = new OutputFormater(Console);
            string Result = Formater.AlignCenter("Hello World!!", 9,'#');
            string Expected = "##Hello##   #World!!#   ";
                          //   ##Hello##___#World!!#___
                          //               New Line
            Assert.AreEqual(Expected, Result);
        }   
        #endregion

    }
}
