﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using PrsiTests.ProgramAbstraction.ConsoleManipulation;
using Prsi.ViewModels;

namespace PrsiTests
{
    [TestClass]
    public class ConfigViewModelTests
    {
        #region PlayerCreation
        [TestMethod]
        public void FivePlayers()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("5");
            OutputFormaterAbstract Formater=new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console,Formater);
            int count=Config.CountOfPlayers;
            Assert.AreEqual(5,count);
        }

        [TestMethod]
        public void ThreePlayers()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("3");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            int count = Config.CountOfPlayers;
            Assert.AreEqual(3, count);
        }

        [TestMethod]
        public void TenPlayers()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("10");
            Console.AddTextToReturn("4");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            int count = Config.CountOfPlayers;
            Assert.AreEqual(4, count);
            Assert.AreEqual("Value is out of range, please repeat", Console.GetOutput(1));
        }

        [TestMethod]
        public void Signed4Players()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("-4");
            Console.AddTextToReturn("4");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            int count = Config.CountOfPlayers;
            Assert.AreEqual(4, count);
            Assert.AreEqual("Value is out of range, please repeat", Console.GetOutput(1));
        }
        #endregion

        #region EnhancedSetOfCardTests
        [TestMethod]
        public void CheckOutput()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("1");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            bool returned = Config.IsEnhancedSetOfCards;
            Assert.AreEqual("We realized, that you have more than 4 players", Console.GetOutput(0));
            Assert.AreEqual("we recommend to use enhanced set of cards with 64 cards", Console.GetOutput(1));
            Assert.AreEqual("1 - Yes", Console.GetOutput(2));
            Assert.AreEqual("2 - No", Console.GetOutput(3));
            Assert.AreEqual("Use enhanced set", Console.GetOutput(4));
        }

        [TestMethod]
        public void SetEnhanced()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("1");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            bool returned = Config.IsEnhancedSetOfCards;
            Assert.AreEqual(true, returned);
        }

        [TestMethod]
        public void DontSetEnhanced()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("2");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            bool returned = Config.IsEnhancedSetOfCards;
            Assert.AreEqual(false, returned);
        }

        [TestMethod]
        public void SetEnhancedInvalid()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("x");
            Console.AddTextToReturn("1");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            bool returned = Config.IsEnhancedSetOfCards;
            Assert.AreEqual(true, returned); 
            Assert.AreEqual("Invalid data, please repeat",Console.GetOutput(5));
        }

        [TestMethod]
        public void SetEnhancedOutOfRange()
        {
            InputAbstract Console = new InputAbstract();
            Console.AddTextToReturn("3");
            Console.AddTextToReturn("1");
            OutputFormaterAbstract Formater = new OutputFormaterAbstract(Console);
            ConfigViewModel Config = new ConfigViewModel(Console, Formater);
            bool returned = Config.IsEnhancedSetOfCards;
            Assert.AreEqual(true, returned);
            Assert.AreEqual("Value is out of range, please repeat", Console.GetOutput(5));
        }
        #endregion
    }
}
