﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Prsi
{
    public class Game
    {
        private const int MIN_COUNT_OF_PLAYERS = 2;
        private const string VALUE_OF_SEVEN = "7";
        private const string VALUE_OF_CHANGING_CARD = "Q";
        private const string VALUE_OF_EMPTY_CARD = " ";

        private int minValueOfCard = 2;
        private int maxValueOfCard = 10;
        private int maxCountOfPlayers = 12;

        private int countOfPlayers;
        private Deck deck;
        private PlayedCards playedCards;
        private List<Player> players;
        private int playerWhoPlay;
        private int winner;

        private List<int> possibilities;
        private RULE rule; 
        private Printer printer;
        private int typKaret;

        private Rules rules;
        private int lowerLimitValuesOfCards;
        private int minPossibilities;

        public Game()
        {
            typKaret = Questions.GetInstance().AnswerInRange("S jakým typem karet chceš hrát?\n0) karty na mariáš\n1) karty na poker (přidávají karty 2-6)\n", 0, 1);
            Console.Clear();

            maxCountOfPlayers = 12;
            lowerLimitValuesOfCards = 2;
            minPossibilities = 0;

            if (typKaret == 0)
            {
                maxCountOfPlayers = 7;
                lowerLimitValuesOfCards = 7;
            }

            countOfPlayers = Questions.GetInstance().AnswerInRange("Zadej počet hráčů (musí být větší, než " + (MIN_COUNT_OF_PLAYERS - 1) + " a menší, než " + (maxCountOfPlayers + 1) + "): ", MIN_COUNT_OF_PLAYERS, maxCountOfPlayers);
            Console.Clear();
            players = Questions.GetInstance().CreatePlayers(countOfPlayers);

            deck = new Deck(lowerLimitValuesOfCards, 10);
            playedCards = new PlayedCards();

            playerWhoPlay = new Random().Next(countOfPlayers);
            winner = -1;
            rule = 0;

            possibilities = new List<int>();

            rules = new Rules(players, playedCards, deck);
            
            rules.DealCards();
            rules.PutCartOnTable(deck.GetTopCardFromDeck());

            printer = new Printer(players, ConsoleColor.Yellow, ConsoleColor.Gray);
        }

        public void Start()
        {
            while (!IsEnd())
            {
                NextRound(playerWhoPlay);

                if (playerWhoPlay == players.Count - 1)
                    playerWhoPlay = 0;
                else
                    playerWhoPlay++;
            }

            Console.Clear();
            if ((winner = GetWinner()) == -1)
                Console.WriteLine("Nastala chyba.");
            else
                Console.WriteLine("Vítězem se stal {0}.", players[winner].Name);
        }

        private void NextRound(int playerWhoPlay)
        {
            Console.Clear();
            printer.PrintError("Na tahu je další hráč: " + players[playerWhoPlay].Name);
            Console.Clear();
            printer.PrintCountOfPlayersCard();
            printer.PrintPlayerWhoPlay(playerWhoPlay);
            printer.PrintOrderOfPlayers(playerWhoPlay, players.Count());
            printer.PrintLastPlayedCard(playedCards);

            possibilities = rules.FindValideCards(playedCards.GetTopCardFromDeck(), players[playerWhoPlay].Hand);
            rule = rules.GetRuleNumber(playedCards.GetTopCardFromDeck());

            printer.PrintAllPossibilities(possibilities, playerWhoPlay, rules.CountOfSevens * rules.PenaltyForSeven, rule, !deck.IsEmpty() || playedCards.Cards.Count() > 1 || rule == RULE.ESO);

            if ((minPossibilities = rules.FindMinNumberOfPossibilities(possibilities.Count, playerWhoPlay)) != -1)
                rules.ProcessAnswer(players[playerWhoPlay].PlaySpecificStrategi(possibilities.Count(), minPossibilities), rule, playerWhoPlay, possibilities);
            //ProcessAnswer(players[playerWhoPlay].PlaySpecificStrategi(possibilities.Count(), 0));

            rules.FillEmptyDeck();
        }

        private bool IsEnd()
        {
            foreach (Player p in players)
            {
                if (p.CountOfCards() == 0)
                {
                    //winner = players.IndexOf(p);
                    return true;
                }
            }

            return false;
        }

        private int GetWinner()
        {
            foreach (Player p in players)
            {
                if (p.CountOfCards() == 0)
                {
                    return players.IndexOf(p);
                }
            }

            return -1;
        }
    }
}
