﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Prsi
{
    /// <summary>
    /// This is class, which print "formated" information of actual game.
    /// </summary>
    public class Printer
    {
        private string[] firstPossibilities = { "stát", "líznout si", "líznout si" };

        private List<Player> players;
        private int countOfPlayers;

        private ConsoleColor numbers;
        public ConsoleColor Numbers { get { return numbers; } set { numbers = value; } }

        private ConsoleColor texts;
        public ConsoleColor Texts { get { return texts; } set { texts = value; } }

        public Printer(List<Player> Players)
        {
            players = Players;
            countOfPlayers = players.Count();

            numbers = ConsoleColor.Gray;
            texts = ConsoleColor.Gray;
        }

        public Printer(List<Player> Players, ConsoleColor numbers, ConsoleColor texts)
        {
            players = Players;
            countOfPlayers = players.Count();

            this.numbers = numbers;
            this.texts = texts;
        }


        public void PrintCountOfPlayersCard()
        {
            Console.WriteLine("Stav karet ostatních hráčů:");

            foreach (Player p in players)
            {
                Console.Write("{0}: ", p.Name);
                PrintWithSpecialColor(p.CountOfCards().ToString(), numbers, texts);
                Console.WriteLine();
            }
            Console.WriteLine();
        }

        public void PrintPlayerWhoPlay(int playerWhoPlay)
        {
            Console.WriteLine("Hraje: {0}", players[playerWhoPlay].Name);
        }

        public void PrintOrderOfPlayers(int playerWhoPlay, int countOfPrintedPlayers)
        {
            int index = playerWhoPlay + 1;

            Console.Write("Následuje: ");
            for (int i = 0; i < countOfPrintedPlayers; i++, index++)
            {
                if (i != 0)
                    Console.Write("> ");

                if (index >= countOfPlayers)
                    index = 0;
                Console.Write("{0} ", players[index].Name);
            }
            Console.WriteLine("\n");
        }

        public void PrintLastPlayedCard(PlayedCards playedCards)
        {
            Console.Write("Přikládáš na: ");
            PrintCard(playedCards.GetTopCardFromDeck());
            Console.WriteLine();
        }

        public void PrintAllPossibilities(List<int> possibilities, int playerWhoPlay, int countOfDrawCards, RULE rule, bool canDraw)
        {
            int index = 0;
            string start = "";

            Console.WriteLine("Možnosti tahu:");

            if (canDraw)
            {
                PrintWithSpecialColor("0", numbers, texts);
                Console.Write(") {0}", firstPossibilities[(int)rule]);

                if (rule == RULE.SEVEN)
                    Console.Write("  {0}", countOfDrawCards);
                Console.WriteLine();
            }


            foreach (Card card in players[playerWhoPlay].Hand)
            {
                if (index < possibilities.Count() && possibilities[index] == players[playerWhoPlay].Hand.IndexOf(card))
                {
                    start = (index + 1).ToString();
                    index++;
                    PrintWithSpecialColor(start, numbers, texts);
                    Console.Write(") ");
                }
                else
                {
                    start = "   ";
                    Console.Write(start);
                }

                PrintCard(card);
            }
        }

        public void PrintPossibleColorsToChange()
        {
            Console.WriteLine("\nZvol barvu, na kterou chceš změnit:");
            for (int i = 0; i < 4; i++)
            {
                PrintWithSpecialColor(i.ToString(), numbers, texts);

                Console.WriteLine(") {0}", (char)(COLOR.SRDCE + i));
            }
        }

        public void PrintError(string error)
        {
            PrintWithSpecialColor(error, numbers, texts);
            Console.Write("\nPro pokračování stiskni libovolnou klávesu: ");
            Console.ReadKey();
        }

        private void ChangeTextColor(ConsoleColor color)
        {
            Console.ForegroundColor = color;
        }

        private void PrintWithSpecialColor(string text, ConsoleColor newColor, ConsoleColor oldColor)
        {
            ChangeTextColor(newColor);
            Console.Write(text);
            ChangeTextColor(oldColor);
        }

        private void PrintCard(Card card)
        {
            Console.Write((char)card.Color);
            try
            {
                int value = Convert.ToInt32(card.Value);
                PrintWithSpecialColor(card.Value, numbers, texts);
                Console.WriteLine();
            }
            catch (Exception e)
            {
                Console.WriteLine(card.Value);
            }
        }
    }
}
