﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Prsi
{
    /// <summary>
    /// This class represent deck of cards.
    /// </summary>
    public class Deck : IDeck
    {
        private const int COUNT_OF_COLORS = 4;

        private int minValueOfCard = 2;
        private int maxValueOfCard = 10;

        protected List<Card> cards;
        public List<Card> Cards { get { return cards; } }

        public Deck()
        {
            CreateDeck();
            cards = ShuffleDeck(cards);
        }

        public Deck(int minValueOfCard, int maxValueOfCard)
        {
            this.minValueOfCard = minValueOfCard;
            this.maxValueOfCard = maxValueOfCard;

            CreateDeck();
            cards = ShuffleDeck(cards);
        }

        private void CreateDeck()
        {
            cards = new List<Card>();

            for (int i = 0; i < COUNT_OF_COLORS; i++)
                AddCardWithSpecificColor(COLOR.SRDCE + i, cards);

        }

        private void AddCardWithSpecificColor(COLOR color, List<Card> cards)
        {
            for (int i = minValueOfCard; i <= maxValueOfCard; i++)
            {
                cards.Add(new Card(i.ToString(), color));
            }
            cards.Add(new Card("A", color));
            cards.Add(new Card("J", color));
            cards.Add(new Card("Q", color));
            cards.Add(new Card("K", color));
        }

        public List<Card> ShuffleDeck(List<Card> cards)
        {
            Random random = new Random();
            HashSet<int> used = new HashSet<int>();
            List<Card> newOrder = new List<Card>();

            for (int i = 0; i < cards.Count; i++)
            {
                int index = random.Next(0, cards.Count);
                if (!used.Contains(index))
                {
                    newOrder.Add(cards[index]);
                    used.Add(index);
                }
                else
                {
                    i--;
                }
            }
            return newOrder;
        }

        public virtual Card GetTopCardFromDeck()
        {
            if (IsEmpty())
                return null;

            Card topCard = cards[0];
            cards.RemoveAt(0);
            return topCard;
        }

        public List<Card> GetXTopCardsFromDeck(int count)
        {
            if (IsEmpty())
                return null;

            List<Card> addedCards = new List<Card>();

            for (int i = 0; i < count; i++)
            {
                addedCards.Add(cards[0]);
                cards.RemoveAt(0);
            }
            return addedCards;
        }

        public bool IsEmpty()
        {
            return cards.Count == 0;
        }

        public void AddCards(List<Card> cards)
        {
            this.cards.AddRange(cards);
        }
    }
}
